-- Ensure that the required libaries are present.
assert(LibStub, "Ranch requires LibStub")
assert(LibStub:GetLibrary("LibDataBroker-1.1", true), "Ranch requires LibDataBroker-1.1")

-- Create reference to the LDB Libarary
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

-- Register the data source.
local dataobj = ldb:NewDataObject("Ranch", {type = "data source", icon = "Interface\\Addons\\Ranch\\Ranch.tga", text = "Ranch"})

-- Create a frame and register the "VARIABLES_LOADED" event on it.
CreateFrame("Frame", "RanchFrame");
RanchFrame:SetScript("OnEvent", function(self, event, ...) getglobal("Ranch_"..event)(event, ...) end);
RanchFrame:RegisterEvent("VARIABLES_LOADED")

if GetLocale() == "deDE" then
	ranch_QGT = "Dieser Gegenstand startet eine Quest."
elseif GetLocale() == "esES" or GetLocale() == "esMX" then
	ranch_QGT = "Este objeto inicia una misin"
elseif GetLocale() == "frFR" then
	ranch_QGT = "Cet objet permet de lancer une qute"
else
	ranch_QGT = "This Item Begins a Quest"
end


-- Event handler for the VARIABLES_LOADED event.
function Ranch_VARIABLES_LOADED(event, ...)

	-- Unregister the VARIABLES_LOADED event. Probably not necessary, but keeps things tidy.
	RanchFrame:UnregisterEvent("VARIABLES_LOADED")
	
	-- Create a GameTooltip for use later. This tooltip can be scanned to detect whether or not an item is BoE.
	CreateFrame( "GameTooltip", "RanchGameTip" );
	RanchGameTip:SetOwner( WorldFrame, "ANCHOR_NONE" );
	RanchGameTip:AddFontStrings(
		RanchGameTip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		RanchGameTip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
		);
	
	-- Create the ranch_perm table if it doesn't exist.
	if ranch_perm == nil then
		ranch_perm = {}
	end
	
	-- Create the ranch_options table if it doesn't exist.
	if ranch_options == nil then
		ranch_options = {}
	end
	
	-- Create the list of types, if it doesn't exist.
	if ranch_options["types"] == nil then
		ranch_options["types"] = {}
	end
	
	-- Create display items option, if it doesn't exist.
	if ranch_options["displayitems"] == nil then
		ranch_options["displayitems"] = true
	end
	
	-- Create merge items option, if it doesn't exist.
	if ranch_options["mergeitems"] == nil then
		ranch_options["mergeitems"] = false
	end
	
	-- Create the quest giving items option, if it doesn't exist.
	if ranch_options["questitems"] == nil then
		ranch_options["questitems"] = true
	end
	
	--Create the LDB text option, if it doesn't exist
	if ranch_options["ldbtext"] == nil then
		ranch_options["ldbtext"] = true
	end
	
	-- Create an empty table to hold any temporary items.
	ranch_temp = {}
	
	-- Set the RanchTradeTypes variable
	RanchTradeTypes = 0
	
	-- Create the About Panel.
	RanchAbout = LibStub("tekKonfig-AboutPanel").new(nil, "Ranch")
	
	-- Create the Items Options
	local RanchItemOptions = CreateFrame("Frame", "RanchItemOpts", InterfaceOptionsFramePanelContainer)
	RanchItemOptions.name = "Item Options"
	RanchItemOptions.parent = "Ranch"
	local title, subtitle = LibStub("tekKonfig-Heading").new(RanchItemOptions, "Ranch Item Options", "This panel controls how Ranch deals with certain item types.")
	
	-- Add Display Items option.
	local displayitems = LibStub("tekKonfig-Checkbox").new(RanchItemOptions, nil, "Display BoE items?", "TOPLEFT", subtitle, "BOTTOMLEFT", -2, -8)
	local checksound = displayitems:GetScript("OnClick")
	displayitems:SetScript("OnClick", function(self) checksound(self); ranch_options["displayitems"] = not ranch_options["displayitems"]; end)
	displayitems:SetChecked(ranch_options["displayitems"])

	-- Add Merge Items option.
	local mergeitems = LibStub("tekKonfig-Checkbox").new(RanchItemOptions, nil, "Merge BoE items?", "TOPLEFT", displayitems, "BOTTOMLEFT", -2, -8)
	local checksound = mergeitems:GetScript("OnClick")
	mergeitems:SetScript("OnClick", function(self) checksound(self); ranch_options["mergeitems"] = not ranch_options["mergeitems"]; end)
	mergeitems:SetChecked(ranch_options["mergeitems"])
	
	-- Add Quest Giving Items option
	local questitems = LibStub("tekKonfig-Checkbox").new(RanchItemOptions, nil, "Show items that give quests?", "TOPLEFT", mergeitems, "BOTTOMLEFT", -2, -8)
	local checksound = questitems:GetScript("OnClick")
	questitems:SetScript("OnClick", function(self) checksound(self); ranch_options["questitems"] = not ranch_options["questitems"]; end)
	questitems:SetChecked(ranch_options["questitems"])

	-- Add Display LDB text option
	local ldbtext = LibStub("tekKonfig-Checkbox").new(RanchItemOptions, nil, "Display LDB text when not tracking anything?", "TOPLEFT", questitems, "BOTTOMLEFT", -2, -8)
	local checksound = ldbtext:GetScript("OnClick")
	ldbtext:SetScript("OnClick", function(self) checksound(self); ranch_options["ldbtext"] = not ranch_options["ldbtext"]; Ranch_BAG_UPDATE(nil); end)
	ldbtext:SetChecked(ranch_options["ldbtext"])

	RanchItemOptions:SetScript("OnShow", function(self) displayitems:SetChecked(ranch_options["displayitems"]); mergeitems:SetChecked(ranch_options["mergeitems"]); questitems:SetChecked(ranch_options["questitems"]) end)
	-- Add Items Options to the Interface Menu.
	InterfaceOptions_AddCategory(RanchItemOptions)
	
	-- Create the Trade Goods options
	local RanchTradeOptions = CreateFrame("Frame", "RanchTradeOpts", InterfaceOptionsFramePanelContainer)
	RanchTradeOptions.name = "Trade Goods"
	RanchTradeOptions.parent = "Ranch"
	local title, subtitle = LibStub("tekKonfig-Heading").new(RanchTradeOptions, "Ranch Trade Goods Options", "This panel controls how Ranch deals with Trade Goods.")
	RanchTradeOptions:SetScript("OnShow", function(RanchTradeOptions)
		if RanchTradeTypes < Ranch_tcount(ranch_options["types"]) then
			RanchTradeTypes = Ranch_tcount(ranch_options["types"])
			local goods, shown, rows
			rows = 0
			for goods, shown in pairs(ranch_options["types"]) do
				if shown then
					local GoodsCheck = LibStub("tekKonfig-Checkbox").new(RanchTradeOptions, nil, "Show "..goods.."?", "TOPLEFT", subtitle, "BOTTOMLEFT", -2, -8 - (rows*26))
					rows = rows + 1
					local checksound = GoodsCheck:GetScript("OnClick")
					GoodsCheck:SetScript("OnClick", function(self) checksound(self); ranch_options["types"][goods] = not ranch_options["types"][goods]; end)
					GoodsCheck:SetChecked(ranch_options["types"][goods])
				end
			end
		end
	end)
	InterfaceOptions_AddCategory(RanchTradeOptions)
	
	-- Register the BAG_UPDATE event.
	RanchFrame:RegisterEvent("BAG_UPDATE");
	
	-- Initialise the slash command handler.
	SLASH_RANCH1 = "/ranch"
	SlashCmdList["RANCH"] = Ranch_Command;
	
end

function Ranch_Command(slashargs)
	local startPos, endPos, firstWord, restOfString = string.find(slashargs, "(%w+)[%s%p]*(.*)");
	if firstWord ~= nil then
		firstWord = strlower(firstWord)
	end
	
	if firstWord == "track" then
		if restOfString == "" or restOfString == nil then
			ranchtrack = nil
		else
			ranchtrack = restOfString
		end
		Ranch_BAG_UPDATE(nil)
	elseif firstWord == "temp" then
		ranch_temp[Ranch_ItemID(restOfString)] = true
	elseif firstWord == "add" then
		if restOfString ~= nil and restOfString ~= "" then
			local itemID = Ranch_ItemID(restOfString)
			ranch_perm[ItemID] = true
		end
	elseif firstWord == "remove" then
		if restOfString ~= nil and restOfString ~= "" then
			local itemID = Ranch_ItemID(restOfString)
			local temptable = {}
			local k,v
			for k,_ in pairs(ranch_perm) do
				if k ~= itemID then
					temptable[k] = true
				end
			end
			ranch_perm = temptable
			temptable = nil
		end
	elseif firstWord == "removeall" then
		ranch_perm = {}
	else
		DEFAULT_CHAT_FRAME:AddMessage("Ranch accepts the following arguments:")
		DEFAULT_CHAT_FRAME:AddMessage("track [itemlink]: Tracks an item in the LDB display")
		DEFAULT_CHAT_FRAME:AddMessage("temp [itemlink]: Tracks an item in the LDB tooltip under the 'Temporary' heading. This item will disappear after you logout.")
		DEFAULT_CHAT_FRAME:AddMessage("add [itemlink]: Tracks an item in the LDB tooltip under the 'Custom' heading. This item will only be removed using the 'remove' or 'removeall' commands.")
		DEFAULT_CHAT_FRAME:AddMessage("remove [itemlink]: Removes an item being tracked in the LDB tooltip under the 'Custom' heading.")
		DEFAULT_CHAT_FRAME:AddMessage("removeall: Removes all items being tracked in the LDB tooltip under the 'Custom' heading.")
	end
end

function Ranch_ItemID(itemLink)
	local found, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.*%]")
	if itemString == nil then
		found, _, itemString = string.find(itemLink, "^c%x+|H(.+)|h%[.*%]")
	end
	if itemString == nil then
		return nil
	else
		local _, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", itemString)
		return itemId
	end
end

function dataobj:OnTooltipShow()
    local i, j
	local ranch_items = {}
	local ranch_boe = {}
	local ranch_qg = {} -- Quest Giving Items
	local ranch_cash = 0
	local ranch_uncost = false
	local ranch_junk = false
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			local ContainerItemLink = GetContainerItemLink(i,j)
			if ContainerItemLink ~= nil then
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemValue = GetItemInfo(ContainerItemLink)			
				local itemId = Ranch_ItemID(itemLink)
				if itemType == "Trade Goods" then
					if ranch_items[itemSubType] == nil then
						ranch_items[itemSubType] = {}
					end
					if ranch_items[itemSubType][itemId] == nil then
						ranch_items[itemSubType][itemId] = GetItemCount(itemLink)
					end
				elseif itemType == "Gem" then
					if ranch_items["Gem"] == nil then
						ranch_items["Gem"] = {}
					end
					if ranch_items["Gem"][itemId] == nil then
						ranch_items["Gem"][itemId] = GetItemCount(itemLink)
					end
				elseif itemRarity == 0 then
					ranch_junk = true
					if itemValue == nil then
						ranch_uncost = true
					else
						ranch_cash = ranch_cash + (itemValue * select(2, GetContainerItemInfo(i,j)))
					end
				elseif itemRarity >=2 then
					RanchGameTip:ClearLines();
					RanchGameTip:SetBagItem(i,j);
					if ((RanchGameTipTextLeft2:GetText() ~= "Soulbound") and (RanchGameTipTextLeft2:GetText() ~= "Quest Item") and RanchGameTipTextLeft2:GetText() ~= "Account Bound") then
						if ranch_options["mergeitems"] then
							ranch_boe[itemId] = itemValue
						else
							ranch_boe[itemLink] = itemValue
						end
					end
				end
				RanchGameTip:ClearLines();
				RanchGameTip:SetBagItem(i,j);
				
				-- Looking for Quest Giving Items
				local iRow
				for iRow = 2, 4 do
					if _G["RanchGameTipTextLeft"..iRow] ~= nil then
						if _G["RanchGameTipTextLeft"..iRow]:GetText() == ranch_QGT then
							tinsert(ranch_qg, itemId)
						end
					end
				end
				
				if ranch_temp[itemId] ~= nil then
					ranch_temp[itemId] = GetItemCount(itemLink)
				end
			end
		end
	end
	
	local itemGroup, Collection, itemId, itemCount
	self:AddLine("Ranch")
	
	if ranch_options["displayitems"] and Ranch_tcount(ranch_boe) ~= 0 then
		self:AddLine(" ")
		self:AddLine("Items")
		for itemDesc, itemCost in pairs(ranch_boe) do
			if ranch_options["mergeitems"] then
				local itemCount = GetItemCount(itemDesc)
				local _, _, _, textcash = Ranch_FormatCash(itemCost * itemCount)
				local _, itemLink = GetItemInfo(itemDesc)
				self:AddDoubleLine(itemLink.." (x"..itemCount..")", textcash)
			else
				local _, _, _, textcash = Ranch_FormatCash(itemCost)
				self:AddDoubleLine(itemDesc, textcash)
			end
		end
	end
	
	if ranch_options["questitems"] and Ranch_tcount(ranch_qg) ~= 0 then
		self:AddLine(" ")
		self:AddLine("Quest Givers")
		for _, itemDesc in ipairs(ranch_qg) do
			local _, itemLink = GetItemInfo(itemDesc)
			self:AddLine(itemLink)
		end
	end
	
	for itemGroup, Collection in pairs(ranch_items) do
		if ranch_options["types"][itemGroup] == nil then
			ranch_options["types"][itemGroup] = true
		end
		if ranch_options["types"][itemGroup] then
			self:AddLine(" ")
			self:AddLine(itemGroup)
			for itemId, itemCount in pairs(Collection) do
				_, itemLink = GetItemInfo(itemId)
				self:AddDoubleLine(itemLink,itemCount)
			end
		end
	end

	if Ranch_tcount(ranch_perm) > 0 then
		self:AddLine(" ")
		self:AddLine("Custom")
		local i,v
		for i,v in pairs(ranch_perm) do
			if v then
				local _, itemLink = GetItemInfo(i)
				self:AddDoubleLine(itemLink, GetItemCount(itemLink))
			end
		end
	end
	
	if Ranch_tcount(ranch_temp) > 0 then
		self:AddLine(" ")
		self:AddLine("Temporary")
		local i,v
		for i,v in pairs(ranch_temp) do
			local _, itemLink = GetItemInfo(i)
			self:AddDoubleLine(itemLink, GetItemCount(itemLink))
		end
	end
	
	if ranch_junk then
		self:AddLine(" ")
		local textcash
		if ranch_uncost and ranch_cost == 0 then
			textcash = "??"
		else
			_, _, _, textcash = Ranch_FormatCash(ranch_cash)
			if ranch_uncost then
				textcash = textcash.."+"
			end
		end
		self:AddDoubleLine("Junk:", textcash)
	end
end	

function Ranch_FormatCash(cash)
	local gold = floor(cash/10000)
	local silver = floor((cash-(gold*10000))/100)
	local copper = cash - (silver * 100) - (gold * 10000)
	local textcash = ""
	if gold > 0 then
		textcash = "|cFFFFFFFF"..gold.."|r|cffffd700g|r|cFFFFFFFF, |r"
	end
	if silver > 0 then
		textcash = textcash.."|cFFFFFFFF"..silver.."|r|cffc7c7cfs|r|cFFFFFFFF, |r"
	end
	textcash = textcash.."|cFFFFFFFF"..copper.."|r|cffeda55fc|r"
	return gold, silver, copper, textcash
end

function Ranch_BAG_UPDATE(...)
	if ranchtrack ~= nil then
		dataobj.text = "|r|"..ranchtrack.." - "..GetItemCount(ranchtrack);
	elseif ranch_options["ldbtext"] then
		dataobj.text = "Ranch";
	else
		dataobj.text = "";
	end
end

 -- tcount: count table members even if they're not indexed by numbers
 -- From the Table Helpers page by Mikk on WoWWiki (http://www.wowwiki.com/Table_Helpers)
 function Ranch_tcount(tab)
   local n=0;
   for _ in pairs(tab) do
     n=n+1;
   end
   return n;
 end
 